#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: market@sandquant.com
# Copyright © 2020-present SandQuant Group. All Rights Reserved.

import sandinvest as si
si.register('wangjiean', 'wuchudan520')


#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com

import configparser
import inspect
import os
import operator
import platform
import urllib.parse

import requests
import pandas as pd

from .error import *


class SandInvestClient:

    _ADDR = "https://api.sandquant.com"
    if platform.system() == 'Windows':
        _DIRNAME = os.getenv("USERPROFILE")
    elif platform.system() == 'Linux':
        _DIRNAME = os.getenv("PATH")
        if _DIRNAME is None:
            _DIRNAME = '/root'
    else:
        _DIRNAME = '/'
    _BASENAME = ".SandInvest"
    _PATH = os.path.join(_DIRNAME, _BASENAME)
    _LoginParams = ["ADDR", "ACCOUNT", "PASSWORD"]

    @classmethod
    def register(cls, account, password):
        _ACCOUNT, _PASSWORD = str(account), str(password)
        _params = dict(zip(cls._LoginParams, [cls._ADDR, _ACCOUNT, _PASSWORD]))
        cfg = configparser.ConfigParser()
        cfg.optionxform = str
        cfg.read_dict(dictionary={cls._BASENAME: _params})
        cfg.write(open(cls._PATH, "w"))
        print("Register Success!")

    @classmethod
    def unregister(cls):
        if os.path.exists(cls._PATH):
            os.remove(cls._PATH)
            print("Unregister Success!")
        else:
            try:
                raise RegisterError
            except RegisterError as err:
                print(err)

    @classmethod
    def auto_login(cls):
        cfg = configparser.ConfigParser()
        cfg.optionxform = str
        if os.path.exists(cls._PATH):
            cfg.read(cls._PATH)
            _params = dict(cfg[cls._BASENAME])
            os.environ.update(_params)
            try:
                cls.validate(_params)
            except Exception as err:
                print(err)
                os.remove(cls._PATH)
                try:
                    raise RegisterWrongError
                except RegisterWrongError as err:
                    print(err)
        else:
            try:
                raise LoginError
            except LoginError as err:
                print(err)
            # print("Missing Login Information")

    @classmethod
    def login(cls, account, password):
        _ACCOUNT, _PASSWORD = str(account), str(password)
        _params = dict(zip(cls._LoginParams, [cls._ADDR, _ACCOUNT, _PASSWORD]))
        os.environ.update(_params)
        cls.validate(_params)

    @classmethod
    def settings(cls):
        return {'account': os.getenv('ACCOUNT'), 'password': os.getenv('PASSWORD')}

    @classmethod
    def validate(cls, params):
        """Validate Login Params 验证登录参数"""
        params = {(_.lower(), __) for _, __ in params.items()}
        endpoint = urllib.parse.urljoin(os.getenv("ADDR"), inspect.getframeinfo(inspect.currentframe()).function)
        response = requests.get(endpoint, params=params)
        result = cls.parse_response(response)
        print(result)

    @classmethod
    def parse_response(cls, result: requests.Response):
        if result.status_code == 200:
            _result = result.json()
            result_type = _result.get('type')
            result_data = _result.get('data')
            if _istype(result_type, pd.DataFrame):
                return pd.DataFrame(result_data)
            if _istype(result_type, str):
                return result_data
            if _istype(result_type, list):
                return result_data
            if _istype(result_type, dict):
                return result_data

        elif result.status_code == 403:
            _error = result.json()['detail'][0]
            _error_msg = _error['msg']
            raise ValueError(_error_msg)

        elif result.status_code == 404:
            _error = result.json()['detail'][0]
            _error_loc = _error['loc'][1]
            _error_msg = _error['msg']
            raise AttributeError(f"{_error_loc} 参数错误，{_error_msg}")

        else:
            raise ClientError

    @classmethod
    def _headers(cls):
        """获取请求头"""
        return dict(zip(cls._LoginParams, operator.itemgetter(*cls._LoginParams)(os.environ)))

    def __init__(self):
        ...

    def __call__(self, func, **_params):
        _headers = self._headers()
        endpoint = urllib.parse.urljoin(os.getenv("ADDR"), func)
        response = requests.get(endpoint, params=_params, headers=_headers)
        result = self.parse_response(response)
        return result

    def __getattr__(self, func, **_params):
        return lambda **_params: self(func, **_params)


def _istype(_type, __type):
    """判断返回的数据类型"""
    return isinstance(eval(_type), __type)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com

__title__ = 'SandInvest'
__description__ = 'SandInvest是一款专注于量化开发的金融数据接口，开始体验吧！'
__version__ = '0.5.0'
__author__ = 'SandQuant'
__author_email__ = 'data@sandquant.com'
__copyright__ = 'Copyright 2020-2022 SandQuant'



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com


class summary:
    code_ = 'code'
    name_ = 'name'
    symbol_ = 'symbol'
    company_cn = 'company_cn'
    company_en = 'company_en'
    list_date_ = 'list_date'
    delist_date_ = 'delist_date'
    market_ = 'market'
    board_ = 'board'
    all_ = [
        code_, name_, symbol_,
        company_cn, company_en, list_date_, delist_date_,
        market_, board_,
    ]


class price:
    open_ = 'open'
    close_ = 'close'
    high_ = 'high'
    low_ = 'low'
    volume_ = 'volume'
    amount_ = 'amount'
    pre_close_ = 'pre_close'
    pct_change_ = 'pct_change'
    vwap_ = 'vwap'
    limit_up_ = 'limit_up'
    limit_down_ = 'limit_down'
    status_ = 'status'
    all_ = [
        open_, close_, high_, low_, volume_, amount_,
        pre_close_, pct_change_, vwap_, limit_up_, limit_down_, status_,
    ]


class shares:
    total_shares_ = 'total_shares'
    float_shares_ = 'float_shares'
    total_cap_ = 'total_cap'
    float_cap_ = 'float_cap'
    turnover_ = 'turnover'
    all_ = [
        total_shares_, float_shares_, total_cap_, float_cap_, turnover_
    ]


class industry:
    code_ = 'code'
    time_ = 'time'
    ind_code_ = 'ind_code'
    ind_name_ = 'ind_name'
    list_date_ = 'list_date'
    source_ = 'source'
    all_ = [
        code_, time_, ind_code_, ind_name_, list_date_, source_
]


class auction:
    close_ = 'close'
    volume_ = 'volume'
    amount_ = 'amount'
    v_s = ['v_s1', 'v_s2', 'v_s3', 'v_s4', 'v_s5',]
    v_b = ['v_b1', 'v_b2', 'v_b3', 'v_b4', 'v_b5',]
    p_s = ['p_s1', 'p_s2', 'p_s3', 'p_s4', 'p_s5',]
    p_b = ['p_b1', 'p_b2', 'p_b3', 'p_b4', 'p_b5',]
    all_ = [close_, volume_, amount_] + v_s + v_b + p_s + p_b



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: market@sandquant.com
# Copyright © 2020-present SandQuant Group. All Rights Reserved.

class LoginError(Exception):

    def __str__(self):
        return "- 缺少登录信息，请使用：`si.register('account', 'password')`\n" \
               "- Missing Login Information, Please use: `si.register('account', 'password')`."


class RegisterError(Exception):

    def __str__(self):
        return "- 登录信息已经注销。\n" \
               "- Login Information is already unregister."


class RegisterWrongError(Exception):

    def __str__(self):
        return "\n" \
               "- 登录信息出错，请使用`si.register('account', 'password')`方法重新注册。 \n" \
               "- Login Information Fail, Please use: `si.register('account', 'password')` again."


class RunOutError(Exception):

    def __str__(self):
        return "- 今日已超出使用额度。 \n" \
               "- Usage limit has been exceeded today."


class ClientError(Exception):

    def __str__(self):
        return "\n" \
               "- 客户端访问出错。 \n" \
               "- Client access error."



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com
import pandas as pd

from .client import SandInvestClient


def description(type_=str):
    type_ = str(type_)
    return SandInvestClient().index(**locals())


def my_account() -> dict:
    return SandInvestClient().my_account(**locals())


def get_calendar(start_date=None, end_date=None, date=None, offset=0, include=True, type_=None):
    type_ = str(type_)
    return SandInvestClient().get_calendar(**locals())


def is_trading(date=None) -> bool:
    result = get_calendar(date=date)
    if result.empty:
        return False
    else:
        return True


def is_break(date=None) -> bool:
    return not is_trading(date=date)


def get_summary(codes=None, start_date=None, end_date=None, date=None, list_date=None, delist_date=None,
                fields=None, include=True, delisted=False) -> pd.DataFrame:
    if isinstance(codes, (list, tuple)):
        codes = ','.join(codes)
    if isinstance(fields, (list, tuple)):
        fields = ','.join(fields)
    return SandInvestClient().get_summary(**locals())


def get_price(codes=None, start_date=None, end_date=None, date=None,
              offset=0, freq='D', include=True, fields=None, skip=False, fill=True, adj=0) -> pd.DataFrame:
    if isinstance(codes, (list, tuple)):
        codes = ','.join(codes)
    if isinstance(fields, (list, tuple)):
        fields = ','.join(fields)
    return SandInvestClient().get_price(**locals())


def get_shares(codes=None, start_date=None, end_date=None, date=None,
               offset=0, include=True, fields=None) -> pd.DataFrame:
    if isinstance(codes, (list, tuple)):
        codes = ','.join(codes)
    if isinstance(fields, (list, tuple)):
        fields = ','.join(fields)
    return SandInvestClient().get_shares(**locals())


def get_industry(codes=None, ind_codes=None, source=None, date=None, fields=None, type_=None):
    type_ = str(type_)
    if isinstance(codes, (list, tuple)):
        codes = ','.join(codes)
    if isinstance(ind_codes, (list, tuple)):
        ind_codes = ','.join(ind_codes)
    if isinstance(fields, (list, tuple)):
        fields = ','.join(fields)
    return SandInvestClient().get_industry(**locals())


def get_auction(codes=None, start_date=None, end_date=None, date=None,
                offset=0, include=True, fields=None, adj=0) -> pd.DataFrame:
    if isinstance(codes, (list, tuple)):
        codes = ','.join(codes)
    if isinstance(fields, (list, tuple)):
        fields = ','.join(fields)
    return SandInvestClient().get_auction(**locals())


def get_method(method, **params):
    return exec(f"SandInvestClient().{method}(**{params})")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com

import os

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

packages = ['sandinvest']
requires = [
    'pandas',
    'requests',
]

info = {}
with open(os.path.join(here, 'sandinvest', '__version__.py'), 'r', encoding='utf-8') as _version:
    exec(_version.read(), info)

with open("README.md", "r", encoding="utf-8") as _readme:
    readme = _readme.read()

setup(
    name=info['__title__'],
    version=info['__version__'],

    description=info['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',

    author=info['__author__'],
    author_email=info['__author_email__'],

    packages=packages,

    install_requires=requires,
    include_package_data=True,
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: Chinese (Simplified)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
    ],
    keywords=['SandInvest', 'Data API'],
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com


suffix_SZ = ('SH', 'SZ')
suffix_SZSE = ('SSE', 'SZSE')
suffix_XSHE = ('XSHG', 'XSHE')


def code_suffix(codes, suffix):
    suffix = dict(zip(*suffix.keys(), *suffix.values()))
    if isinstance(codes, str):
        for _, __ in suffix.items():
            codes = codes.replace(_, __)
    if isinstance(codes, list):
        codes = [code_suffix(code, suffix) for code in codes]
    return codes



#!/usr/bin/python
# -*- coding: utf-8 -*-
# @Author : SandQuant
# @Email: data@sandquant.com

from .client import SandInvestClient
from .api import *
from .utils import *
from .fields import *

from .__version__ import __title__, __description__, __copyright__
from .__version__ import __version__, __author__, __author_email__

auto_login = SandInvestClient.auto_login
unregister = SandInvestClient.unregister
settings = SandInvestClient.settings


def login(account, password):
    return SandInvestClient.login(**locals())


def register(account, password):
    return SandInvestClient.register(**locals())


auto_login()
